' *** LCD16x2CharTest.bas *********** 2009.06.10 *********** Ron Hackett ***
'
' This program runs on the 14M-based 16x2 LCD board @ 8 MHz. It simply
' displays two columns of HD44780 characters to check for a match with the
' Table in the HD44780 data sheet.
'
' Because we are using DB5..DB2 rather than DB7..DB4, high nibbles need to
' to be divided by 4 & low nibbles multiplied by 4 to place on correct pins.
'
' **************************************************************************


' *** Constants *********

	symbol cmd    = 0				' used to set up for cmd/txt byte		
	symbol txt    = 1				' used to set up for cmd/txt byte
	symbol enable = 1				' LCD enable pin = PICAXE output1
	symbol RSin   = outpin0		' LCD RegSel pin = PICAXE output0


' *** Variables *********

	symbol char  = b0				' character to be sent to LCD
	symbol index = b1				' used in for/next loops
	symbol RSbit = b2				' used to set up for cmd/txt byte
	symbol temp  = b3				' used to manipulate character data


' *** Directives ********

	'#com2							' set COM port for downloading
	#picaxe14M						' set compiler mode


' *** Begin Main Program **************************************************
 	
	setfreq m8

' *** Initialize the LCD ***

	pins =  0						' clear all output lines
	pause 200						' pause 200 mS for LCD initialization

	pins = 12						' (48/4=12) set to 4-bit operation
 	pulsout enable,1				' send data
	pause 10							' pause 10 mS
	pulsout enable,1				' send again
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins =  8						' (32/4=8) set to 4-bit operation
	pulsout enable,1				' send data.
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins = 32						' (128/4=32) set to 2 line operation
	pulsout enable,1				' send data

	char = 12						' display on, cursor off
	RSbit = cmd
	gosub OutByte					' send instruction to LCD


'	Display text on LCD

	char  = 128						' place cursor at beginning of line 1
	RSbit = cmd	
	gosub OutByte
	
	for index = 224 to 239		' display next-to-last column from Table
		char = index
		gosub OutByte
	next index	
	
	char  = 192						' place cursor at beginning of line 2
	RSbit = cmd	
	gosub OutByte
	
	for index = 240 to 255 		' display last column from Table
		char = index
		gosub OutByte
	next index
	
	end


' *** End Main Program - Subroutines Follow ******************************
	


' *** OutByte Subroutine ***

OutByte:
	temp = char / 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place high nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
		
	temp = char * 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place low nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
	
	RSbit = txt						' reset for text display

	return






