' *** LCD16x2CustCharDemo2.bas ******* 2009.06.10 ******* Ron Hackett ***
'
'  Program runs on a Picaxe-14M @ 4 MHz & sends data to an HD44780 16x2 LCD.
'	Since we are using DB5...DB2 rather than DB7...DB4, high nibbles need to
'	be divided by 4 and low nibbles multiplied by 4 to place on correct pins.
'
'  Program demonstrates the use of custom characters on an HD44780 display
'  by defining 5 custom lower-case letters with true descenders.
'
' ***********************************************************************


' *** Constants *********

	symbol cmd    = 0				' used to set up for cmd/txt byte		
	symbol txt    = 1				' used to set up for cmd/txt byte
	symbol enable = 1				' LCD enable pin = PICAXE output1
	symbol RSin   = outpin0		' LCD RegSel pin = PICAXE output0


' *** Variables *********

	symbol char  = b1				' character to be sent to LCD
	symbol index = b2				' used in for/next loops
	symbol RSbit = b3				' used to set up for cmd/txt byte
	symbol temp  = b4				' used to manipulate character data



' *** Directives ********

	'#com2							' set COM port for downloading
	#picaxe14M						' set compiler mode


' *** Data ***

'	Lowercase "g"
	Data (%00000)
	Data (%00000)
	Data (%01111)	'  ****
	Data (%10001)	' *   *
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *
	Data (%01110)	'  ***

'	Lowercase "p"
	Data (%00000)
	Data (%00000)
	Data (%10110)	' * **
	Data (%11001)	' **  *
	Data (%10001)	' *   *
	Data (%11110)	' ****
	Data (%10000)	' *
	Data (%10000)	' *

'	Lowercase "q"
	Data (%00000)
	Data (%00000)
	Data (%01101)	'  ** *	
	Data (%10011)	' *  **	
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *	
	Data (%00001)	'     *

'	Lowercase "j"	
	Data (%00010)	'    *
	Data (%00000)	'
	Data (%00110)	'   **	
	Data (%00010)	'    *	
	Data (%00010)	'    *
	Data (%00010)	'    *	
	Data (%10010)	' *  *	
	Data (%01100)	'  **	

'	Lowercase "y"	
	Data (%00000)
	Data (%00000)
	Data (%10001)	' *	*
	Data (%10001)	' *   *	
	Data (%10001)	' *   *
	Data (%01111)	'  ****	
	Data (%00001)	'     *	
	Data (%01110)	'  ***
	
'	Backslash
	Data (%00000)
	Data (%10000)	' *
	Data (%01000)	'  *
	Data (%00100)	'   *
	Data (%00010)	'    *
	Data (%00001)	'     *
	Data (%00000)
	Data (%00000)

'	Bullet
	Data (%00000)
	Data (%01110)	'  ***
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%01110)	'  ***
	Data (%00000)
	Data (%00000)	
	
'	Heart
	Data (%00000)
	Data (%01010)	'  * *
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%01110)	'  *** 
	Data (%00100)	'   *
	Data (%00000)
	Data (%00000)

	Data 64,("A jolly pig quit")
	Data 80,("A ",3,"oll",4," ",1,"i",0," ",2,"uit")


' *** Begin Main Program ************************************************
 	
	setfreq m8


' *** Initialize the LCD ***

	pins =  0						' clear all output lines
	pause 200						' pause 200 mS for LCD initialization

	pins = 12						' (48/4=12) set to 4-bit operation
 	pulsout enable,1				' send data
	pause 10							' pause 10 mS
	pulsout enable,1				' send again
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins =  8						' (32/4=8) set to 4-bit operation
	pulsout enable,1				' send data.
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins = 32						' (128/4=32) set to 2 line operation
	pulsout enable,1				' send data

	char = 12						' display on, cursor off
	RSbit = cmd
	gosub OutByte					' send instruction to LCD
	

'*** Load custom chars into CGRAM ***

	char  = 64						' command: define custom characters
	RSbit = cmd
	gosub OutByte
	
	for index = 0 to 63			' install 5 custom characters
		read index, char
		gosub OutByte
	next index


'*** Display text on LCD ***

	char  = 128						' place cursor at beginning of line 1
	RSbit = cmd	
	gosub OutByte
	
	for index = 64 to 79			' display default characters 
		read index, char
		gosub OutByte
	next index	
	
	char  = 192						' place cursor at beginning of line 2
	RSbit = cmd	
	gosub OutByte
	
	for index = 80 to 95			' display custom characters
		read index, char
		gosub OutByte
	next index
	
	end


' *** End Main Program - Subroutines Follow *****************************
	


' *** OutByte Subroutine ***

OutByte:
	temp = char / 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place high nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
		
	temp = char * 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place low nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
	
	RSbit = txt						' reset for text display

	return







