' *** LCD16x2-CustCharDriver.bas ****** 2009.06.19 ****** Ron Hackett ***
'
' This program runs on a Picaxe-14M @ 8 MHz. It receives serial data from
' the master processor & diplays it on an HD 44780-type 16x2 LCD. Because
' we are using DB5...DB2 rather than DB7...DB4, high nibbles need to be
' divided by 4 and low nibbles multiplied by 4 to place on correct pins.
'
' Program may appear crude, but it is fast. By dedicating 9 variables to
' the reception of serial data, it avoids the necessity of requiring the
' master processor to delay between the transmission of each character.
'
' NOTE: In order for the LCD to function correctly, the master processor
' must send serial data in "packets" of nine characters. The first char
' must be a command that specifies the display location at which printing
' should begin. The remaining 8 characters will usually be text to print.
' (Two exceptions to this rule are discussed in the 8/09 PICAXE Primer.)
'
' ***********************************************************************


' *** Constants *********

	symbol cmd    = 0					' used to set up for cmd/txt byte		
	symbol txt    = 1					' used to set up for cmd/txt byte
	symbol enable = 1					' LCD enable pin = PICAXE output1
	symbol RSin   = outpin0			' LCD RegSel pin = PICAXE output0


' *** Variables *********

	symbol char  = b9					' character to be sent to LCD
	symbol index = b10				' used in for/next loop
	symbol RSbit = b11				' used to set up for cmd/txt byte
	symbol temp  = b12				' used to manipulate character data


' *** Directives ********

	'#com2								' set COM port for downloading
	#picaxe14M							' set compiler mode


' *** Data ***							' Keep them in this order.
											' The program requires it!
'	Lowercase "g"
	Data (%00000)
	Data (%00000)
	Data (%01111)	'  ****
	Data (%10001)	' *   *
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *
	Data (%01110)	'  ***

'	Lowercase "p"
	Data (%00000)
	Data (%00000)
	Data (%10110)	' * **
	Data (%11001)	' **  *
	Data (%10001)	' *   *
	Data (%11110)	' ****
	Data (%10000)	' *
	Data (%10000)	' *

'	Lowercase "q"
	Data (%00000)
	Data (%00000)
	Data (%01101)	'  ** *	
	Data (%10011)	' *  **	
	Data (%10001)	' *   *
	Data (%01111)	'  ****
	Data (%00001)	'     *	
	Data (%00001)	'     *

'	Lowercase "j"	
	Data (%00010)	'    *
	Data (%00000)	'    
	Data (%00110)	'	 **
	Data (%00010)	'    *	
	Data (%00010)	'    *
	Data (%00010)	'    *
	Data (%10010)	' *  *	
	Data (%01100)	'  **	

'	Lowercase "y"	
	Data (%00000)
	Data (%00000)
	Data (%10001)	' *	*
	Data (%10001)	' *   *	
	Data (%10001)	' *   *
	Data (%01111)	'  ****	
	Data (%00001)	'     *	
	Data (%01110)	'  ***
	
'	Bullet
	Data (%00000)
	Data (%01110)	'  ***
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%11111)	' *****
	Data (%01110)	'  ***
	Data (%00000)
	Data (%00000)


' *** Begin Main Program ************************************************
 	
	setfreq m8


' *** Initialize the LCD ***

	pins =  0						' clear all output lines
	pause 200						' pause 200 mS for LCD initialization

	pins = 12						' (48/4=12) set to 4-bit operation
 	pulsout enable,1				' send data
	pause 10							' pause 10 mS
	pulsout enable,1				' send again
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins =  8						' (32/4=8) set to 4-bit operation
	pulsout enable,1				' send data.
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins = 32						' (128/4=32) set to 2 line operation
	pulsout enable,1				' send data

	char = 12						' display on, cursor off
	RSbit = cmd
	gosub OutCmnd					' send instruction to LCD
	

'*** Load custom chars into CGRAM ***

	char  = 64						' command: define custom characters
	RSbit = cmd
	gosub OutCmnd
	
	for index = 0 to 47			' install 6 custom characters
		read index, char
		gosub OutText
	next index


' *** Main loop: Receive data and display it. ***

do
	serin 4,N4800_8,b0,b1,b2,b3,b4,b5,b6,b7,b8

	char  = b0
	RSbit = cmd
	gosub OutCmnd
		
	char = b1
	gosub OutText
			
	char = b2
	gosub OutText
				
	char = b3
	gosub OutText
				
	char = b4
	gosub OutText
			
	char = b5
	gosub OutText
			
	char = b6
	gosub OutText
		
	char = b7
	gosub OutText
	
	char = b8
	gosub OutText	
loop
	

' *** End Main Program - Subroutines Follow *****************************
	


' *** OutText and OutCmnd Subroutines ***

OutText:

		select case char
			case 103,112,113,106		' "g", "p", "q" & "j"
				char = char - 103
			case 121						' "y"
				char = 4
			case 128 to 207			' provide capability to move the
				RSbit = cmd				' cursor to any on-screen location
		endselect

OutCmnd:
	temp = char / 4					' because we're using DB5...DB2
	temp = temp & %00111100			' mask for  DB5...DB2
	pins = temp							' place high nibble of char onto pins
	RSin = RSbit						' RSin = 0 for command or 1 for text
	pulsout enable,1					' send data
		
	temp = char * 4					' because we're using DB5...DB2
	temp = temp & %00111100			' mask for  DB5...DB2
	pins = temp							' place low nibble of char onto pins
	RSin = RSbit						' RSin = 0 for command or 1 for text
	pulsout enable,1					' send data
	
	RSbit = txt							' reset for text display
	
	return


