' *** LCD16x2Scroll.bas ************ 2009.06.10 ************ Ron Hackett ***
'
'  Program runs on a Picaxe-14M @ 4 MHz & sends data to an HD44780 16x2 LCD.
'	Since we are using DB5...DB2 rather than DB7...DB4, high nibbles need to
'	be divided by 4 and low nibbles multiplied by 4 to place on correct pins.
'
'  The program demonstrates display scrolling on an HD44780 LCD display.
'
' **************************************************************************


' *** Constants ***

	symbol   abit = 700			' used for pauses
	symbol    cmd =   0			' used to set up for "command" byte type			
	symbol    txt =   1			' used to set up for "text" byte type
	symbol enable =   1			' LCD enable pin connected to PICAXE output1
	symbol   RSin = outpin0		' LCD RegSel pin connected to PICAXE output0


' *** Variables ***

	symbol  char = b0				' character to be sent to LCD
	symbol index = b1				' used as counter in inner for/next loops
	symbol  loup = b2				' used as counter in outer For/next loops
	symbol RSbit = b3				' used to set up for cmd/txt byte type
	symbol  temp = b4				' used to manipulate char without destruction


' *** Directives ***

	'#com2
	#picaxe 14M


' *** Data ***

	data 0,("The five boxing wizards jumped quickly. ")
  
  
' *** Begin Main Program ***************************************************
 	
	setfreq m8
	
' *** Initialize the LCD ***

	pins =  0						' clear all output lines
	pause 200						' pause 200 mS for LCD initialization

	pins = 12						' (48/4=12) set to 4-bit operation
 	pulsout enable,1				' send data
	pause 10							' pause 10 mS
	pulsout enable,1				' send again
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins =  8						' (32/4=8) set to 4-bit operation
	pulsout enable,1				' send data.
	pulsout enable,1				' send again (necessary in 4-bit mode)
	
	pins = 32						' (128/4=32) set to 2 line operation
	pulsout enable,1				' send data

	char = 12						' display on, cursor off
	RSbit = cmd
	gosub OutByte					' send instruction to LCD


  	char = 1
  	RSbit = cmd					`		' clear display and go home	
 	gosub OutByte					' send instruction to LCD
 	
 	wait 1

	for index = 0 to 39			' output data to 1st row of LCD
		read index, char			' read character from EEPROM
		gosub OutByte				' send character to LCD
	next index
	
	wait 2

do
	for loup = 1 to 3
	  	for index = 0 to 39
	    	char = 24
	    	RSbit = cmd
	    	gosub OutByte
	    	pause abit
	  	next index
	next loup
	
	 wait 2

	for loup = 1 to 3
	  for index = 0 to 39
	    char = 28
	    RSbit = cmd
	    gosub OutByte
	    pause abit
	  next index
	next loup
		  
	wait 2
	  
loop
 	

' *** End Main Program - Subroutines Follow ********************************
	


' *** OutByte Subroutine ***

OutByte:
	temp = char / 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place high nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
		
	temp = char * 4				' because we're using DB5...DB2
	temp = temp & %00111100		' mask for  DB5...DB2
	pins = temp						' place low nibble of char onto pins
	RSin = RSbit					' RSin = 0 for command or 1 for text
	pulsout enable,1				' send data
	
	RSbit = txt						' reset for text display

	return


